import { createElement, createContext, useState, useContext, useRef, useCallback, useEffect, memo, Fragment } from 'react';
import { string, oneOfType, object, array, bool, oneOf, number, func } from 'prop-types';
import { createPortal } from 'react-dom';
import { ResizeSensor } from 'css-element-queries';

var Centered = function (props) { return (createElement("div", { className: "spaces-centered" }, props.children)); };
var CenteredVertically = function (props) { return (createElement("div", { className: "spaces-centered-vertically" }, props.children)); };

var ClearFix = function (props) { return (createElement("div", { className: "spaces-clearfix" }, props.children)); };

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var SpaceContext = createContext(null);
var SpaceLayerContext = createContext(null);
var SpaceInfoContext = createContext(null);

var ResizeType;
(function (ResizeType) {
    ResizeType["Left"] = "resize-left";
    ResizeType["Right"] = "resize-right";
    ResizeType["Top"] = "resize-top";
    ResizeType["Bottom"] = "resize-bottom";
})(ResizeType || (ResizeType = {}));
var AnchorType;
(function (AnchorType) {
    AnchorType["Left"] = "anchor-left";
    AnchorType["Right"] = "anchor-right";
    AnchorType["Top"] = "anchor-top";
    AnchorType["Bottom"] = "anchor-bottom";
})(AnchorType || (AnchorType = {}));
var AnchorTypes = [
    AnchorType.Left,
    AnchorType.Right,
    AnchorType.Bottom,
    AnchorType.Top
];
var AnchorToResizeTypeMap = {
    "anchor-left": ResizeType.Right,
    "anchor-right": ResizeType.Left,
    "anchor-bottom": ResizeType.Top,
    "anchor-top": ResizeType.Bottom
};
var CenterType;
(function (CenterType) {
    CenterType["None"] = "none";
    CenterType["Vertical"] = "vertical";
    CenterType["HorizontalVertical"] = "horizontalVertical";
})(CenterType || (CenterType = {}));
var publicProps = {
    id: string,
    className: string,
    style: oneOfType([object, array]),
    scrollable: bool,
    trackSize: bool,
    centerContent: oneOf([CenterType.None, CenterType.Vertical, CenterType.HorizontalVertical]),
    as: string,
    zIndex: number,
    onClick: func,
    onMouseDown: func,
    onMouseEnter: func,
    onMouseLeave: func,
    onMouseMove: func,
    onTouchStart: func,
    onTouchMove: func,
    onTouchEnd: func
};
var privateProps = {
    anchorSize: oneOfType([string, number]),
    anchor: oneOf([AnchorType.Bottom, AnchorType.Left, AnchorType.Right, AnchorType.Top]),
    resizable: bool,
    order: number,
    topMost: bool
};
var anchoredProps = {
    size: oneOfType([string, number]).isRequired,
    order: number
};
var resizableProps = {
    handleSize: number,
    overlayHandle: bool,
    minimumSize: number,
    maximumSize: number,
    onResizeStart: func,
    onResizeEnd: func
};
var positionedProps = {
    left: oneOfType([string, number]),
    top: oneOfType([string, number]),
    right: oneOfType([string, number]),
    bottom: oneOfType([string, number]),
    width: oneOfType([string, number]),
    height: oneOfType([string, number]),
    resizable: bool
};
var allProps = __assign({}, publicProps, privateProps, resizableProps, positionedProps);

var getSizeString = function (size) { return typeof (size) === "string" ? size : "" + size + (size !== 0 ? "px" : ""); };
var isHorizontalSpace = function (anchorType) {
    return anchorType && (anchorType === AnchorType.Left || anchorType === AnchorType.Right);
};
var isVerticalSpace = function (anchorType) {
    return anchorType && (anchorType === AnchorType.Top || anchorType === AnchorType.Bottom);
};
function shortuuid() {
    var firstPart = (Math.random() * 46656) | 0;
    var secondPart = (Math.random() * 46656) | 0;
    return ("000" + firstPart.toString(36)).slice(-3) + ("000" + secondPart.toString(36)).slice(-3);
}
var initialState = function (props) { return ({
    id: props.id || "s" + shortuuid(),
    children: []
}); };
var cssValue = function (value, adjusted) {
    return adjusted ?
        "calc(" + getSizeString(value || 0) + " + " + getSizeString(adjusted) + ")" :
        getSizeString(value || 0);
};

var recalcSpaces = function (spaces) {
    var _loop_1 = function (sidx) {
        var space = spaces[sidx];
        var adjustedTop = [];
        var adjustedLeft = [];
        var adjustedRight = [];
        var adjustedBottom = [];
        if (space.anchorType === undefined) {
            space.left = 0;
            space.top = 0;
            space.right = 0;
            space.bottom = 0;
        }
        else {
            space.left = space.anchorType !== AnchorType.Right ? 0 : undefined;
            space.top = space.anchorType !== AnchorType.Bottom ? 0 : undefined;
            space.right = space.anchorType !== AnchorType.Left ? 0 : undefined;
            space.bottom = space.anchorType !== AnchorType.Top ? 0 : undefined;
        }
        var _loop_2 = function (at) {
            var spaceTakers = spaces
                .filter(function (t) { return t.anchorType !== undefined && t.zIndex === space.zIndex && t.anchorType === AnchorTypes[at]; })
                .sort(function (a, b) { return a.order - b.order; });
            for (var i = 0; i < spaceTakers.length; i++) {
                var t = spaceTakers[i];
                if (t.id !== space.id) {
                    var adjustedSize = t.adjustedSize !== 0 ? " + " + getSizeString(t.adjustedSize) : "";
                    if (space.anchorType === undefined) {
                        if (t.anchorType === AnchorType.Top && t.anchorSize) {
                            adjustedTop.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Left && t.anchorSize) {
                            adjustedLeft.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Bottom && t.anchorSize) {
                            adjustedBottom.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Right && t.anchorSize) {
                            adjustedRight.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                    }
                    else {
                        if (t.anchorType === AnchorType.Top && t.anchorSize && space.top !== undefined) {
                            adjustedTop.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Left && t.anchorSize && space.left !== undefined) {
                            adjustedLeft.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Bottom && t.anchorSize && space.bottom !== undefined) {
                            adjustedBottom.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                        else if (t.anchorType === AnchorType.Right && t.anchorSize && space.right !== undefined) {
                            adjustedRight.push("" + getSizeString(t.anchorSize) + adjustedSize);
                        }
                    }
                }
                else {
                    break;
                }
            }
        };
        for (var at = 0; at < AnchorTypes.length; at++) {
            _loop_2(at);
        }
        [
            { adjusted: adjustedTop, setter: function (value) { return space.top = value; } },
            { adjusted: adjustedBottom, setter: function (value) { return space.bottom = value; } },
            { adjusted: adjustedLeft, setter: function (value) { return space.left = value; } },
            { adjusted: adjustedRight, setter: function (value) { return space.right = value; } }
        ].map(function (x) { return x.adjusted.length > 0 && x.setter("calc(" + x.adjusted.join(" + ") + ")"); });
    };
    for (var sidx = 0; sidx < spaces.length; sidx++) {
        _loop_1(sidx);
    }
    return spaces;
};
var registerSpace = function (context, space) {
    var existing = context.children.find(function (t) { return t.id === space.id; });
    if (!existing) {
        context.updateChildren(recalcSpaces(context.children.concat([space])));
        return space;
    }
    else {
        existing.order = space.order;
        existing.anchorType = space.anchorType;
        existing.anchorSize = space.anchorSize;
        return existing;
    }
};
var removeSpace = function (context, id) {
    context.updateChildren(recalcSpaces(context.children.filter(function (t) { return t.id !== id; })));
};
var updateSpace = function (context, id, delta) {
    context.updateChildren(recalcSpaces(context.children.map(function (t) { return t.id === id ? __assign({}, t, delta) : t; })));
};
var createSpaceContext = function (children, updateChildren, currentSpace, parent) {
    var context = {
        level: parent ? parent.level + 1 : 0,
        children: children,
        updateChildren: updateChildren
    };
    return context;
};

var HeadStyles = function (props) {
    var spaces = props.spaces;
    if (spaces.length > 0) {
        var styles = [];
        for (var i = 0; i < spaces.length; i++) {
            var space = spaces[i];
            var css = [];
            var style = {
                left: (space.left !== undefined ? cssValue(space.left, space.adjustedLeft) : undefined),
                top: (space.top !== undefined ? cssValue(space.top, space.adjustedTop) : undefined),
                right: (space.right !== undefined ? cssValue(space.right, space.adjustedLeft) : undefined),
                bottom: (space.bottom !== undefined ? cssValue(space.bottom, space.adjustedTop) : undefined),
                width: isHorizontalSpace(space.anchorType) ? cssValue(space.anchorSize, space.adjustedSize) : space.width,
                height: isVerticalSpace(space.anchorType) ? cssValue(space.anchorSize, space.adjustedSize) : space.height,
                zIndex: space.zIndex
            };
            if (style.left) {
                css.push("left: " + style.left + ";");
            }
            if (style.top) {
                css.push("top: " + style.top + ";");
            }
            if (style.right) {
                css.push("right: " + style.right + ";");
            }
            if (style.bottom) {
                css.push("bottom: " + style.bottom + ";");
            }
            if (style.width) {
                css.push("width: " + style.width + ";");
            }
            if (style.height) {
                css.push("height: " + style.height);
            }
            if (style.zIndex) {
                css.push("z-index: " + style.zIndex);
            }
            if (css.length > 0) {
                styles.push("#" + space.id + " { " + css.join(' ') + " }");
            }
        }
        if (styles.length === 0) {
            return null;
        }
        return createPortal(createElement("style", null, styles.join(' ')), window.document.head);
    }
    return null;
};

var Fixed = function (props) {
    var _a = useState([]), children = _a[0], setChildren = _a[1];
    var style = __assign({}, props.style, {
        width: props.width,
        height: props.height
    });
    return (createElement("div", { className: "spaces-fixedsize-layout" + (props.className ? " " + props.className : ""), style: style },
        createElement(HeadStyles, { spaces: children }),
        createElement(SpaceContext.Provider, { value: createSpaceContext(children, setChildren) }, props.children)));
};
Fixed.propTypes = {
    className: string,
    style: oneOfType([object, array]),
    width: number,
    height: number.isRequired
};

var ViewPort = function (props) {
    var _a = useState([]), children = _a[0], setChildren = _a[1];
    return (createElement("div", { className: "spaces-fullpage-layout" + (props.className ? " " + props.className : ""), style: {
            left: props.left || 0,
            top: props.top || 0,
            right: props.right || 0,
            bottom: props.bottom || 0
        } },
        createElement(HeadStyles, { spaces: children }),
        createElement(SpaceContext.Provider, { value: createSpaceContext(children, setChildren) }, props.children)));
};
ViewPort.propTypes = {
    className: string,
    left: number,
    top: number,
    right: number,
    bottom: number
};

var Layer = function (props) {
    return createElement(SpaceLayerContext.Provider, { value: props.zIndex }, props.children);
};

var Info = function (props) { return (createElement(SpaceInfoContext.Consumer, null, function (info) { return props.children(info); })); };

var useSpace = function (props, divElementRef) {
    var parentContext = useContext(SpaceContext);
    if (!parentContext) {
        throw new Error("No top-level space: There must be a <ViewPort /> or <Fixed /> ancestor space");
    }
    var _a = useState(initialState(props)), state = _a[0], changeState = _a[1];
    var _b = useState(null), currentSize = _b[0], setCurrentSize = _b[1];
    var resizeSensor = useRef();
    var onRemove = useRef();
    var setState = function (stateDelta) { return changeState(function (prev) { return (__assign({}, prev, stateDelta)); }); };
    var layerContext = useContext(SpaceLayerContext);
    var currentZIndex = props.zIndex || layerContext || 0;
    var updateCurrentSize = useCallback(function () {
        if (divElementRef.current) {
            var rect = divElementRef.current.getBoundingClientRect();
            setCurrentSize({
                width: rect.width,
                height: rect.height
            });
        }
    }, []);
    var space = registerSpace(parentContext, {
        id: state.id,
        zIndex: currentZIndex,
        order: props.order === undefined ? 1 : props.order,
        anchorType: props.anchor,
        anchorSize: props.anchorSize || 0,
        adjustedSize: 0,
        adjustedLeft: 0,
        adjustedTop: 0,
        width: props.anchor && isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
        height: props.anchor && isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height
    });
    useEffect(function () {
        setCurrentSize(null);
    }, [parentContext.children.length]);
    // Deal with property changes to size / zIndex / anchoring 
    useEffect(function () {
        updateSpace(parentContext, state.id, {
            width: isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
            height: isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height,
            adjustedSize: 0
        });
    }, [props.anchorSize]);
    useEffect(function () {
        updateSpace(parentContext, state.id, {
            left: props.anchor !== AnchorType.Right ? props.left || 0 : undefined,
            top: props.anchor !== AnchorType.Bottom ? props.top || 0 : undefined,
            right: props.anchor !== AnchorType.Left ? props.right || 0 : undefined,
            bottom: props.anchor !== AnchorType.Top ? props.bottom || 0 : undefined,
            width: isHorizontalSpace(props.anchor) ? props.anchorSize || 0 : props.width,
            height: isVerticalSpace(props.anchor) ? props.anchorSize || 0 : props.height
        });
    }, [props.left, props.top, props.bottom, props.right, props.width, props.height, props.anchor]);
    useEffect(function () {
        updateSpace(parentContext, state.id, {
            zIndex: currentZIndex
        });
    }, [currentZIndex]);
    // Setup / cleanup
    useEffect(function () {
        if (divElementRef.current) {
            if (props.trackSize) {
                resizeSensor.current = new ResizeSensor(divElementRef.current, function (size) { return setCurrentSize({
                    width: size.width,
                    height: size.height
                }); });
            }
        }
        updateCurrentSize();
        var cleanup = function () {
            resizeSensor.current && resizeSensor.current.detach();
            onRemove.current && onRemove.current();
        };
        return cleanup;
    }, []);
    useEffect(function () {
        if (!currentSize) {
            updateCurrentSize();
        }
    });
    onRemove.current = useCallback(function () {
        removeSpace(parentContext, state.id);
    }, []);
    var currentContext = createSpaceContext(state.children, function (children) { return setState({ children: children }); }, space, parentContext);
    return {
        space: space,
        parentContext: parentContext,
        currentContext: currentContext,
        currentSize: currentSize || { width: 0, height: 0 }
    };
};

var ResizeHandle = function (props) {
    return (createElement("div", { style: { width: props.width, height: props.height }, className: "spaces-resize-handle " + props.type, onMouseDown: props.onMouseDown, onTouchStart: props.onTouchStart }));
};
ResizeHandle.propTypes = {
    type: oneOf([ResizeType.Bottom, ResizeType.Left, ResizeType.Right, ResizeType.Top]).isRequired,
    width: number,
    height: number
};

function throttle(callback, limit) {
    var wait = false; // Initially, we're not waiting
    return function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        if (!wait) { // If we're not waiting
            callback.apply(void 0, args); // Execute users function
            wait = true; // Prevent future invocations
            setTimeout(function () {
                wait = false; // And allow future invocations
            }, limit);
        }
    };
}

var RESIZE_THROTTLE = 5;
var Fill = function (props) { return createElement(SpaceInternal, __assign({}, props)); };
Fill.propTypes = publicProps;
var Top = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Top, anchorSize: props.size })); };
Top.propTypes = __assign({}, publicProps, anchoredProps);
var Left = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Left, anchorSize: props.size })); };
Left.propTypes = __assign({}, publicProps, anchoredProps);
var Bottom = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Bottom, anchorSize: props.size })); };
Bottom.propTypes = __assign({}, publicProps, anchoredProps);
var Right = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Right, anchorSize: props.size })); };
Right.propTypes = __assign({}, publicProps, anchoredProps);
var TopResizable = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Top, anchorSize: props.size, resizable: true })); };
TopResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var LeftResizable = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Left, anchorSize: props.size, resizable: true })); };
LeftResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var BottomResizable = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Bottom, anchorSize: props.size, resizable: true })); };
BottomResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var RightResizable = function (props) { return createElement(SpaceInternal, __assign({}, props, { anchor: AnchorType.Right, anchorSize: props.size, resizable: true })); };
RightResizable.propTypes = __assign({}, publicProps, anchoredProps, resizableProps);
var Positioned = function (props) { return createElement(SpaceInternal, __assign({}, props)); };
RightResizable.propTypes = __assign({}, publicProps, resizableProps, positionedProps);
var SpaceInternal = memo(function (props) {
    var divElementRef = useRef();
    var _a = useSpace(props, divElementRef), space = _a.space, parentContext = _a.parentContext, currentContext = _a.currentContext, currentSize = _a.currentSize;
    var handleSize = props.handleSize === undefined ? 5 : props.handleSize;
    var overlayHandle = props.overlayHandle !== undefined ? props.overlayHandle : true;
    var resizeType = undefined;
    var resizeHandle = undefined;
    var onResizeEnd = useCallback(function () {
        if (divElementRef.current) {
            var currentRect = divElementRef.current.getBoundingClientRect();
            props.onResizeEnd && props.onResizeEnd(Math.floor(resizeType === ResizeType.Left || resizeType === ResizeType.Right ? currentRect.width : currentRect.height));
        }
    }, []);
    if (parentContext && props.anchor && props.resizable) {
        resizeType = AnchorToResizeTypeMap[props.anchor];
        var resizeHandleWidth = resizeType === ResizeType.Left || resizeType === ResizeType.Right ? handleSize : undefined;
        var resizeHandleHeight = resizeType === ResizeType.Top || resizeType === ResizeType.Bottom ? handleSize : undefined;
        var onResize_1 = function (originalX, originalY, x, y, minimumAdjust, maximumAdjust) {
            var adjustmentX = Math.min(Math.max(resizeType === ResizeType.Left ? originalX - x : x - originalX, minimumAdjust), maximumAdjust === undefined ? 999999 : maximumAdjust);
            var adjustmentY = Math.min(Math.max(resizeType === ResizeType.Top ? originalY - y : y - originalY, minimumAdjust), maximumAdjust === undefined ? 999999 : maximumAdjust);
            var adjustment = isHorizontalSpace(props.anchor) ? adjustmentX : adjustmentY;
            if (adjustment !== space.adjustedSize) {
                updateSpace(parentContext, space.id, { adjustedSize: adjustment });
            }
        };
        var startTouchResize_1 = function (e) {
            if (!divElementRef.current) {
                return;
            }
            var rect = divElementRef.current.getBoundingClientRect();
            var size = isHorizontalSpace(props.anchor) ? rect.width : rect.height;
            var originalTouchX = resizeType === ResizeType.Left ? e.touches[0].pageX + space.adjustedSize : e.touches[0].pageX - space.adjustedSize;
            var originalTouchY = resizeType === ResizeType.Top ? e.touches[0].pageY + space.adjustedSize : e.touches[0].pageY - space.adjustedSize;
            var minimumAdjust = (props.minimumSize === undefined ? 20 : props.minimumSize) - size + space.adjustedSize;
            var maximumAdjust = props.maximumSize ? (props.maximumSize - size + space.adjustedSize) : undefined;
            var lastX = 0;
            var lastY = 0;
            var moved = false;
            var touchResize = function (x, y) { return onResize_1(originalTouchX, originalTouchY, x, y, minimumAdjust, maximumAdjust); };
            var throttledTouchResize = throttle(touchResize, RESIZE_THROTTLE);
            var withPreventDefault = function (e) {
                moved = true;
                lastX = e.touches[0].pageX;
                lastY = e.touches[0].pageY;
                e.preventDefault();
                e.stopImmediatePropagation();
                throttledTouchResize(lastX, lastY);
            };
            var removeListener = function () {
                if (moved) {
                    touchResize(lastX, lastY);
                }
                window.removeEventListener('touchmove', withPreventDefault);
                window.removeEventListener('touchend', removeListener);
                onResizeEnd();
            };
            window.addEventListener('touchmove', withPreventDefault);
            window.addEventListener('touchend', removeListener);
            e.preventDefault();
            e.stopPropagation();
            props.onResizeStart && props.onResizeStart();
        };
        var startResize_1 = function (e) {
            if (!divElementRef.current) {
                return;
            }
            var rect = divElementRef.current.getBoundingClientRect();
            var size = isHorizontalSpace(props.anchor) ? rect.width : rect.height;
            var originalMouseX = resizeType === ResizeType.Left ? e.pageX + space.adjustedSize : e.pageX - space.adjustedSize;
            var originalMouseY = resizeType === ResizeType.Top ? e.pageY + space.adjustedSize : e.pageY - space.adjustedSize;
            var minimumAdjust = (props.minimumSize === undefined ? 20 : props.minimumSize) - size + space.adjustedSize;
            var maximumAdjust = props.maximumSize ? (props.maximumSize - size + space.adjustedSize) : undefined;
            var lastX = 0;
            var lastY = 0;
            var moved = false;
            var mouseResize = function (x, y) { return onResize_1(originalMouseX, originalMouseY, x, y, minimumAdjust, maximumAdjust); };
            var throttledMouseResize = throttle(mouseResize, RESIZE_THROTTLE);
            var withPreventDefault = function (e) {
                moved = true;
                lastX = e.pageX;
                lastY = e.pageY;
                e.preventDefault();
                e.stopImmediatePropagation();
                throttledMouseResize(lastX, lastY);
            };
            var removeListener = function () {
                if (moved) {
                    mouseResize(lastX, lastY);
                }
                window.removeEventListener('mousemove', withPreventDefault);
                window.removeEventListener('mouseup', removeListener);
                onResizeEnd();
            };
            window.addEventListener('mousemove', withPreventDefault);
            window.addEventListener('mouseup', removeListener);
            e.preventDefault();
            e.stopPropagation();
            props.onResizeStart && props.onResizeStart();
        };
        resizeHandle =
            createElement(ResizeHandle, { type: resizeType, adjustedSize: space.adjustedSize, width: resizeHandleWidth, height: resizeHandleHeight, onMouseDown: function (e) { return startResize_1(e); }, onTouchStart: function (e) { return startTouchResize_1(e); } });
    }
    var outerStyle = {
        left: (space.left !== undefined ? cssValue(space.left, space.adjustedLeft) : undefined),
        top: (space.top !== undefined ? cssValue(space.top, space.adjustedTop) : undefined),
        right: (space.right !== undefined ? cssValue(space.right, space.adjustedLeft) : undefined),
        bottom: (space.bottom !== undefined ? cssValue(space.bottom, space.adjustedTop) : undefined),
        width: isHorizontalSpace(props.anchor) ? cssValue(props.anchorSize, space.adjustedSize) : space.width,
        height: isVerticalSpace(props.anchor) ? cssValue(props.anchorSize, space.adjustedSize) : space.height,
        zIndex: space.zIndex
    };
    var innerStyle = __assign({}, props.style, {
        left: resizeType === ResizeType.Left && !overlayHandle ? handleSize : undefined,
        top: resizeType === ResizeType.Top && !overlayHandle ? handleSize : undefined,
        right: resizeType === ResizeType.Right && !overlayHandle ? handleSize : undefined,
        bottom: resizeType === ResizeType.Bottom && !overlayHandle ? handleSize : undefined
    });
    var userClasses = props.className ?
        props.className.split(' ').map(function (c) { return c.trim(); }) :
        [];
    var outerClasses = [
        "spaces-space",
        props.scrollable ? (resizeHandle ? "scrollable" : "scrollable-a") : undefined
    ].concat((resizeHandle && props.scrollable ? userClasses.map(function (c) { return c + "-container"; }) : userClasses)).filter(function (c) { return c; });
    var innerClasses = [
        "spaces-space-inner"
    ].concat(userClasses).filter(function (c) { return c; });
    var children = props.children;
    if (props.centerContent === CenterType.Vertical) {
        children = createElement(CenteredVertically, null, children);
    }
    else if (props.centerContent === CenterType.HorizontalVertical) {
        children = createElement(Centered, null, children);
    }
    return (resizeHandle && props.scrollable ?
        createElement(props.as || 'div', {
            id: space.id,
            ref: divElementRef,
            className: outerClasses.join(' '),
            style:  undefined,
            onClick: props.onClick,
            onMouseDown: props.onMouseDown,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave,
            onMouseMove: props.onMouseMove,
            onTouchStart: props.onTouchStart,
            onTouchMove: props.onTouchMove,
            onTouchEnd: props.onTouchEnd
        }, createElement(Fragment, null,
             createElement(HeadStyles, { spaces: currentContext.children }),
            resizeHandle,
            createElement("div", { className: innerClasses.join(' '), style: innerStyle },
                createElement(SpaceContext.Provider, { value: currentContext },
                    createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(currentSize.width), height: Math.floor(currentSize.height) } }, children))))) :
        createElement(props.as || 'div', {
            id: space.id,
            ref: divElementRef,
            className: outerClasses.join(' '),
            style:  innerStyle,
            onClick: props.onClick,
            onMouseDown: props.onMouseDown,
            onMouseEnter: props.onMouseEnter,
            onMouseLeave: props.onMouseLeave,
            onMouseMove: props.onMouseMove,
            onTouchStart: props.onTouchStart,
            onTouchMove: props.onTouchMove,
            onTouchEnd: props.onTouchEnd
        }, createElement(Fragment, null,
             createElement(HeadStyles, { spaces: currentContext.children }),
            resizeHandle,
            createElement(SpaceContext.Provider, { value: currentContext },
                createElement(SpaceInfoContext.Provider, { value: { width: Math.floor(currentSize.width), height: Math.floor(currentSize.height) } }, children)))));
});
SpaceInternal.propTypes = allProps;

var useParentSpace = function () {
    return {
        startDrag: function () { return null; }
    };
};

export { AnchorType, Bottom, BottomResizable, CenterType, Centered, CenteredVertically, ClearFix, Fill, Fixed, Info, Layer, Left, LeftResizable, Positioned, Right, RightResizable, Top, TopResizable, ViewPort, useParentSpace };
//# sourceMappingURL=server.js.map
